panathinaikos_green = "#004d00"
panathinaikos_white = "#FFFFFF"
fenerbahce_blue = "#09183D"
fenerbahce_yellow = "#FFD700"
real_madrid_blue = "#00529B"
real_madrid_gold = "#FDB913"
olympiacos_red = "#E30613"
olympiacos_white = "#FFFFFF"
barcelona_blue = "#004d98"
barcelona_yellow = "#ffcc00"
barcelona_purple = "#a50044"
red_star_red = "red"
red_star_blue = "blue"
monaco_red = "#C6111B"
monaco_orange = "#C07A12"
basconia_blue = "#0B1931"
basconia_red = "#CE092A"
zalgiris_green = "#006633"
efes_blue = "#005CA9"
milano_red = "#D40026"
valencia_orange = "#FE6C0E"
valencia_black = "#5D5D5E"
milano_red = "#E0261D"
efes_blue = "#1340B7"
efes_red = "#FE3348"
zalgiris_green = "#0A6E3D"
zalgiris_orange = "#F9832B"
bayern_red = "#CC0000"
bayern_gold = "#FDB913"
bayern_blue = "#0066B2"
asvel_pink = "#EFBDDA"
asvel_gray = "#929292"
alba_yellow = "#FED000"
alba_blue = "#04305C"
alba_light_blue = "#77C8F1"
maccabi_yellow = "#FEF471"
maccabi_blue = "#2470B5"
paris_blue = "#3873BA"
paris_red = "#E73621"
paris_black = "#000000"
