team_configs = {
    "ULK": {
        "background": {"color": "#0033A0"},
        "line": {"color": "#FFD700", "width": 4, "alpha": 0.5, "style": "-"},
        "markers": {"s": 40, "marker": "s", "color": "white", "alpha": 0.8},
        "labels": {"color": "#FFD700", "fontsize": 18},
        "names": {
            "color": "#FFD700",
            "fontsize": 16,
            "fontweight": "bold",
            "rotation": -30,
            "ha": "center",
        },
        "axes": {"color": "#FFD700", "fontsize": 18, "width": 4},
    },
    # Add other team configurations here
    "RED": {
        "background": {"color": "red"},
        "line": {"color": "white", "width": 4, "alpha": 0.5, "style": "-"},
        "markers": {"s": 40, "marker": "s", "color": "white", "alpha": 0.8},
        "labels": {"color": "white", "fontsize": 18},
        "names": {
            "color": "white",
            "fontsize": 16,
            "fontweight": "bold",
            "rotation": 0,
            "ha": "center",
        },
        "axes": {"color": "white", "fontsize": 18, "width": 4},
    },
    "PAN": {
        "background": {"color": "#004d00"},  # Dark green background
        "line": {"color": "white", "width": 4, "alpha": 0.5, "style": "-"},
        "markers": {"s": 40, "marker": "o", "color": "white", "alpha": 0.8},
        "labels": {"color": "white", "fontsize": 18},
        "names": {
            "color": "white",
            "fontsize": 16,
            "fontweight": "bold",
            "rotation": 0,
            "ha": "center",
        },
        "axes": {"color": "white", "fontsize": 18, "width": 4},
    },
    "MAD": {
        "background": {"color": "#ffffff"},  # White background
        "line": {
            "color": "#00529B",
            "width": 4,
            "alpha": 0.5,
            "style": "-",
        },  # Blue for line
        "markers": {
            "s": 40,
            "marker": "o",
            "color": "#FDB913",
            "alpha": 0.8,
        },  # Gold for markers
        "labels": {"color": "#00529B", "fontsize": 18},  # Blue for labels
        "names": {
            "color": "#00529B",
            "fontsize": 16,
            "fontweight": "bold",
            "rotation": 0,
            "ha": "center",
        },  # Blue for names
        "axes": {"color": "#00529B", "fontsize": 18, "width": 4},  # Blue for axes
    },
    "BAS": {
        "background": {"color": "#002244"},  # Dark blue background
        "line": {
            "color": "#c8102e",
            "width": 4,
            "alpha": 0.5,
            "style": "-",
        },  # Red for line
        "markers": {
            "s": 40,
            "marker": "o",
            "color": "#c8102e",
            "alpha": 0.8,
        },  # Red for markers
        "labels": {"color": "#c8102e", "fontsize": 18},
        "names": {
            "color": "#c8102e",
            "fontsize": 16,
            "fontweight": "bold",
            "rotation": 0,
            "ha": "center",
        },
        "axes": {"color": "#c8102e", "fontsize": 18, "width": 4},
    },
    "BAR": {
        "background": {"color": "#004d98"},  # Dark blue background
        "line": {
            "color": "#ffcc00",
            "width": 4,
            "alpha": 0.5,
            "style": "-",
        },  # Yellow for line
        "markers": {
            "s": 40,
            "marker": "o",
            "color": "#a50044",
            "alpha": 0.8,
        },  # Red for markers
        "labels": {"color": "#ffcc00", "fontsize": 18},  # Yellow for labels
        "names": {
            "color": "#ffffff",
            "fontsize": 16,
            "fontweight": "bold",
            "rotation": 0,
            "ha": "center",
        },  # White for names
        "axes": {"color": "#ffffff", "fontsize": 18, "width": 4},  # White for axes
    },
    "ASV": {
        "background": {"color": "black"},
        "line": {
            "color": "#ffffff",
            "width": 4,
            "alpha": 0.5,
            "style": "-",
        },  # White for line
        "markers": {
            "s": 40,
            "marker": "o",
            "color": "#ffffff",
            "alpha": 0.8,
        },  # White for markers
        "labels": {"color": "#ffffff", "fontsize": 18},
        "names": {
            "color": "#ffffff",
            "fontsize": 16,
            "fontweight": "bold",
            "rotation": 0,
            "ha": "center",
        },
        "axes": {"color": "#ffffff", "fontsize": 18, "width": 4},
    },
    "MUN": {
        "background": {"color": "#CC0000"},  # Dark red background
        "line": {
            "color": "#0066B2",
            "width": 4,
            "alpha": 0.5,
            "style": "-",
        },  # Blue for line
        "markers": {
            "s": 40,
            "marker": "o",
            "color": "#FDB913",
            "alpha": 0.8,
        },  # Gold for markers
        "labels": {"color": "#0066B2", "fontsize": 18},  # Blue for labels
        "names": {
            "color": "#FDB913",
            "fontsize": 16,
            "fontweight": "bold",
            "rotation": 0,
            "ha": "center",
        },  # Gold for names
        "axes": {"color": "#0066B2", "fontsize": 18, "width": 4},  # Blue for axes
    },
    "BER": {
        "background": {"color": "#ffcc00"},  # Yellow background
        "line": {
            "color": "#0033a0",
            "width": 4,
            "alpha": 0.5,
            "style": "-",
        },  # Blue for line
        "markers": {
            "s": 40,
            "marker": "o",
            "color": "#0033a0",
            "alpha": 0.8,
        },  # Blue for markers
        "labels": {"color": "#0033a0", "fontsize": 18},
        "names": {
            "color": "#0033a0",
            "fontsize": 16,
            "fontweight": "bold",
            "rotation": 0,
            "ha": "center",
        },
        "axes": {"color": "#0033a0", "fontsize": 18, "width": 4},
    },
    "PAR": {
        "background": {"color": "black"},
        "line": {"color": "white", "width": 4, "alpha": 0.5, "style": "-"},
        "markers": {"s": 40, "marker": "o", "color": "white", "alpha": 0.8},
        "labels": {"color": "white", "fontsize": 18},
        "names": {
            "color": "white",
            "fontsize": 16,
            "fontweight": "bold",
            "rotation": 0,
            "ha": "center",
        },
        "axes": {"color": "white", "fontsize": 18, "width": 4},
    },
    "TEL": {
        "background": {"color": "#0033a0"},  # Blue background
        "line": {
            "color": "#ffd700",
            "width": 4,
            "alpha": 0.5,
            "style": "-",
        },  # Gold for line
        "markers": {
            "s": 40,
            "marker": "o",
            "color": "#ffd700",
            "alpha": 0.8,
        },  # Gold for markers
        "labels": {"color": "#ffd700", "fontsize": 18},
        "names": {
            "color": "#ffd700",
            "fontsize": 16,
            "fontweight": "bold",
            "rotation": 0,
            "ha": "center",
        },
        "axes": {"color": "#ffd700", "fontsize": 18, "width": 4},
    },
    "ZAL": {
        "background": {"color": "#006633"},  # Green background
        "line": {
            "color": "#ffffff",
            "width": 4,
            "alpha": 0.5,
            "style": "-",
        },  # White for line
        "markers": {
            "s": 40,
            "marker": "o",
            "color": "#ffffff",
            "alpha": 0.8,
        },  # White for markers
        "labels": {"color": "#ffffff", "fontsize": 18},
        "names": {
            "color": "#ffffff",
            "fontsize": 16,
            "fontweight": "bold",
            "rotation": 0,
            "ha": "center",
        },
        "axes": {"color": "#ffffff", "fontsize": 18, "width": 4},
    },
    "OLY": {
        "background": {"color": "#FFFFFF"},  # White background
        "line": {
            "color": "#E30613",
            "width": 4,
            "alpha": 0.5,
            "style": "-",
        },  # Red for line
        "markers": {
            "s": 40,
            "marker": "o",
            "color": "#FFD700",
            "alpha": 0.8,
        },  # Gold for markers
        "labels": {"color": "#E30613", "fontsize": 18},  # Red for labels
        "names": {
            "color": "#E30613",
            "fontsize": 16,
            "fontweight": "bold",
            "rotation": 0,
            "ha": "center",
        },  # Red for names
        "axes": {"color": "#E30613", "fontsize": 18, "width": 4},  # Red for axes
    },
    "VIR": {
        "background": {"color": "#000000"},  # Black background
        "line": {
            "color": "#FFD700",
            "width": 4,
            "alpha": 0.5,
            "style": "-",
        },  # Gold for line
        "markers": {
            "s": 40,
            "marker": "o",
            "color": "#FFFFFF",
            "alpha": 0.8,
        },  # White for markers
        "labels": {"color": "#FFD700", "fontsize": 18},  # Gold for labels
        "names": {
            "color": "#FFFFFF",
            "fontsize": 16,
            "fontweight": "bold",
            "rotation": 0,
            "ha": "center",
        },  # White for names
        "axes": {"color": "#FFD700", "fontsize": 18, "width": 4},  # Gold for axes
    },
    "MIL": {
        "background": {"color": "#D40026"},  # Red background
        "line": {
            "color": "black",
            "width": 4,
            "alpha": 0.5,
            "style": "-",
        },  # White for line
        "markers": {
            "s": 40,
            "marker": "o",
            "color": "#000000",
            "alpha": 0.8,
        },  # Black for markers
        "labels": {"color": "#FFFFFF", "fontsize": 18},  # White for labels
        "names": {
            "color": "#FFFFFF",
            "fontsize": 16,
            "fontweight": "bold",
            "rotation": 0,
            "ha": "center",
        },  # Black for names
        "axes": {"color": "#FFFFFF", "fontsize": 18, "width": 4},  # White for axes
    },
    "IST": {
        "background": {"color": "#005CA9"},  # Blue background
        "line": {
            "color": "#FFFFFF",
            "width": 4,
            "alpha": 0.5,
            "style": "-",
        },  # White for line
        "markers": {
            "s": 40,
            "marker": "o",
            "color": "#FF0000",
            "alpha": 0.8,
        },  # Red for markers
        "labels": {"color": "#FFFFFF", "fontsize": 18},  # White for labels
        "names": {
            "color": "#FFFFFF",
            "fontsize": 16,
            "fontweight": "bold",
            "rotation": 0,
            "ha": "center",
        },  # Red for names
        "axes": {"color": "#FFFFFF", "fontsize": 18, "width": 4},  # White for axes
    },
    "PAM": {
        "background": {"color": "#FF6600"},  # Orange background
        "line": {
            "color": "#000000",
            "width": 4,
            "alpha": 0.5,
            "style": "-",
        },  # Black for line
        "markers": {
            "s": 40,
            "marker": "o",
            "color": "#FFFFFF",
            "alpha": 0.8,
        },  # White for markers
        "labels": {"color": "#000000", "fontsize": 18},  # Black for labels
        "names": {
            "color": "#FFFFFF",
            "fontsize": 16,
            "fontweight": "bold",
            "rotation": 0,
            "ha": "center",
        },  # White for names
        "axes": {"color": "#000000", "fontsize": 18, "width": 4},  # Black for axes
    },
    "MCO": {
        "background": {"color": "#8B0000"},  # Dark red background
        "line": {
            "color": "#FFFFFF",
            "width": 4,
            "alpha": 0.5,
            "style": "-",
        },  # White for line
        "markers": {
            "s": 40,
            "marker": "o",
            "color": "#FFD700",
            "alpha": 0.8,
        },  # Gold for markers
        "labels": {"color": "#FFFFFF", "fontsize": 18},  # White for labels
        "names": {
            "color": "#FFD700",
            "fontsize": 16,
            "fontweight": "bold",
            "rotation": 0,
            "ha": "center",
        },  # Gold for names
        "axes": {"color": "#FFFFFF", "fontsize": 18, "width": 4},  # White for axes
    },
}
