from basket_viz.config.euroleague_color_config import *

team_configs = {
    "RED": {
        "court_line_color": "white",
        "line_width": 2,
        "outer_lines": True,
        "court_background_color": "red",  # Panathinaikos green
        "color_map": {"made": "white", "miss": "black"},
        "marker_size": 8,
        "figsize": (14, 12),
        "coord_x": "COORD_X",
        "coord_y": "COORD_Y",
        "sort_col": "UTC",
        "animation_interval": 100,
        "animation_repeat_delay": 1000,
        "animation_blit": True,
        "title": {"fontsize": 24, "fontweight": "bold", "color": "white"},
    },
    "OLY": {
        "court_line_color": olympiacos_red,
        "line_width": 2,
        "outer_lines": True,
        "court_background_color": olympiacos_white,  # Panathinaikos green
        "color_map": {"made": olympiacos_red, "miss": "black"},
        "marker_size": 8,
        "figsize": (14, 12),
        "title": {"fontsize": 24, "fontweight": "bold", "color": olympiacos_red},
        #'made_action_ids': ['LAYUPMD', '3FGM', '2FGM', 'DUNK', ],
        #'miss_action_ids': ['3FGA', 'LAYUPATT','2FGA', '3FGAB', '2FGAB'],
        "sort_col": "UTC",
        "coord_x": "COORD_X",
        "coord_y": "COORD_Y",
        "animation_interval": 100,
        "animation_repeat_delay": 1000,
        "animation_blit": True,
    },
    "ULK": {
        "court_line_color": fenerbahce_yellow,
        "line_width": 2,
        "outer_lines": True,
        "court_background_color": fenerbahce_blue,  # Panathinaikos green
        "color_map": {"made": fenerbahce_yellow, "miss": "white"},
        "marker_size": 8,
        "figsize": (14, 12),
        "title": {"fontsize": 24, "fontweight": "bold", "color": fenerbahce_yellow},
        #'made_action_ids': ['LAYUPMD', '3FGM', '2FGM', 'DUNK', ],
        #'miss_action_ids': ['3FGA', 'LAYUPATT','2FGA', '3FGAB', '2FGAB'],
        "sort_col": "UTC",
        "coord_x": "COORD_X",
        "coord_y": "COORD_Y",
        "sort_col": "UTC",
        "animation_interval": 100,
        "animation_repeat_delay": 1000,
        "animation_blit": True,
    },
    "MAD": {
        "court_line_color": real_madrid_blue,
        "line_width": 2,
        "outer_lines": True,
        "court_background_color": "white",  # Panathinaikos green
        "color_map": {"made": real_madrid_gold, "miss": real_madrid_blue},
        "marker_size": 8,
        "figsize": (14, 12),
        #'made_action_ids': ['LAYUPMD', '3FGM', '2FGM', 'DUNK', ],
        #'miss_action_ids': ['3FGA', 'LAYUPATT','2FGA', '3FGAB', '2FGAB'],
        "sort_col": "UTC",
        "coord_x": "COORD_X",
        "coord_y": "COORD_Y",
        "sort_col": "UTC",
        "animation_interval": 100,
        "animation_repeat_delay": 1000,
        "animation_blit": True,
        "title": {"fontsize": 24, "fontweight": "bold", "color": real_madrid_blue},
    },
    "PAN": {
        "court_line_color": panathinaikos_white,
        "line_width": 2,
        "outer_lines": True,
        "court_background_color": panathinaikos_green,  # Panathinaikos green
        "color_map": {"made": panathinaikos_white, "miss": "lightgray"},
        "marker_size": 8,
        "figsize": (14, 12),
        "coord_x": "COORD_X",
        "coord_y": "COORD_Y",
        "sort_col": "UTC",
        "animation_interval": 100,
        "animation_repeat_delay": 1000,
        "animation_blit": True,
        "title": {"fontsize": 24, "fontweight": "bold", "color": "white"},
    },
    "BAR": {
        "court_line_color": barcelona_yellow,
        "line_width": 2,
        "outer_lines": True,
        "court_background_color": barcelona_blue,  # Panathinaikos green
        "color_map": {"made": barcelona_purple, "miss": barcelona_yellow},
        "marker_size": 8,
        "figsize": (14, 12),
        "coord_x": "COORD_X",
        "coord_y": "COORD_Y",
        "sort_col": "UTC",
        "animation_interval": 100,
        "animation_repeat_delay": 1000,
        "animation_blit": True,
        "title": {"fontsize": 24, "fontweight": "bold", "color": barcelona_yellow},
    },
    "MIL": {
        "court_line_color": milano_red,
        "line_width": 2,
        "outer_lines": True,
        "court_background_color": "black",  # Panathinaikos green
        "color_map": {"made": milano_red, "miss": "white"},
        "marker_size": 8,
        "figsize": (14, 12),
        "coord_x": "COORD_X",
        "coord_y": "COORD_Y",
        "sort_col": "UTC",
        "animation_interval": 100,
        "animation_repeat_delay": 1000,
        "animation_blit": True,
        "title": {"fontsize": 24, "fontweight": "bold", "color": milano_red},
    },
    "PAM": {
        "court_line_color": "white",
        "line_width": 2,
        "outer_lines": True,
        "court_background_color": valencia_orange,  # Panathinaikos green
        "color_map": {"made": "black", "miss": "white"},
        "marker_size": 8,
        "figsize": (14, 12),
        "coord_x": "COORD_X",
        "coord_y": "COORD_Y",
        "sort_col": "UTC",
        "animation_interval": 100,
        "animation_repeat_delay": 1000,
        "animation_blit": True,
        "title": {"fontsize": 24, "fontweight": "bold", "color": "black"},
    },
    "PAR": {
        "court_line_color": "white",
        "line_width": 2,
        "outer_lines": True,
        "court_background_color": "black",  # Panathinaikos green
        "color_map": {"made": "white", "miss": "lightgray"},
        "marker_size": 8,
        "figsize": (14, 12),
        "coord_x": "COORD_X",
        "coord_y": "COORD_Y",
        "sort_col": "UTC",
        "animation_interval": 100,
        "animation_repeat_delay": 1000,
        "animation_blit": True,
        "title": {"fontsize": 24, "fontweight": "bold", "color": "white"},
    },
    "MCO": {
        "court_line_color": "white",
        "line_width": 2,
        "outer_lines": True,
        "court_background_color": monaco_orange,
        "color_map": {"made": monaco_red, "miss": "white"},
        "marker_size": 8,
        "figsize": (14, 12),
        "coord_x": "COORD_X",
        "coord_y": "COORD_Y",
        "sort_col": "UTC",
        "animation_interval": 100,
        "animation_repeat_delay": 1000,
        "animation_blit": True,
        "title": {"fontsize": 24, "fontweight": "bold", "color": monaco_red},
    },
    "BAS": {
        "court_line_color": basconia_red,
        "line_width": 2,
        "outer_lines": True,
        "court_background_color": basconia_blue,  # Panathinaikos green
        "color_map": {"made": basconia_red, "miss": "white"},
        "marker_size": 8,
        "figsize": (14, 12),
        "coord_x": "COORD_X",
        "coord_y": "COORD_Y",
        "sort_col": "UTC",
        "animation_interval": 100,
        "animation_repeat_delay": 1000,
        "animation_blit": True,
        "title": {"fontsize": 24, "fontweight": "bold", "color": basconia_red},
    },
    "VIR": {
        "court_line_color": "white",
        "line_width": 2,
        "outer_lines": True,
        "court_background_color": "black",
        "color_map": {"made": "red", "miss": "white"},
        "marker_size": 8,
        "figsize": (14, 12),
        #'made_action_ids': ['LAYUPMD', '3FGM', '2FGM', 'DUNK', ],
        #'miss_action_ids': ['3FGA', 'LAYUPATT','2FGA', '3FGAB', '2FGAB'],
        "sort_col": "UTC",
        "coord_x": "COORD_X",
        "coord_y": "COORD_Y",
        "sort_col": "UTC",
        "animation_interval": 100,
        "animation_repeat_delay": 1000,
        "animation_blit": True,
        "title": {"fontsize": 24, "fontweight": "bold", "color": "white"},
    },
    "IST": {
        "court_line_color": "white",
        "line_width": 2,
        "outer_lines": True,
        "court_background_color": efes_blue,
        "color_map": {"made": efes_red, "miss": "white"},
        "marker_size": 8,
        "figsize": (14, 12),
        #'made_action_ids': ['LAYUPMD', '3FGM', '2FGM', 'DUNK', ],
        #'miss_action_ids': ['3FGA', 'LAYUPATT','2FGA', '3FGAB', '2FGAB'],
        "sort_col": "UTC",
        "coord_x": "COORD_X",
        "coord_y": "COORD_Y",
        "sort_col": "UTC",
        "animation_interval": 100,
        "animation_repeat_delay": 1000,
        "animation_blit": True,
        "title": {"fontsize": 24, "fontweight": "bold", "color": "white"},
    },
    "ZAL": {
        "court_line_color": "white",
        "line_width": 2,
        "outer_lines": True,
        "court_background_color": zalgiris_green,
        "color_map": {"made": "white", "miss": zalgiris_orange},
        "marker_size": 8,
        "figsize": (14, 12),
        #'made_action_ids': ['LAYUPMD', '3FGM', '2FGM', 'DUNK', ],
        #'miss_action_ids': ['3FGA', 'LAYUPATT','2FGA', '3FGAB', '2FGAB'],
        "sort_col": "UTC",
        "coord_x": "COORD_X",
        "coord_y": "COORD_Y",
        "sort_col": "UTC",
        "animation_interval": 100,
        "animation_repeat_delay": 1000,
        "animation_blit": True,
        "title": {"fontsize": 24, "fontweight": "bold", "color": "white"},
    },
    "MUN": {
        "court_line_color": bayern_blue,
        "line_width": 2,
        "outer_lines": True,
        "court_background_color": bayern_red,
        "color_map": {"made": "white", "miss": bayern_blue},
        "marker_size": 8,
        "figsize": (14, 12),
        #'made_action_ids': ['LAYUPMD', '3FGM', '2FGM', 'DUNK', ],
        #'miss_action_ids': ['3FGA', 'LAYUPATT','2FGA', '3FGAB', '2FGAB'],
        "sort_col": "UTC",
        "coord_x": "COORD_X",
        "coord_y": "COORD_Y",
        "sort_col": "UTC",
        "animation_interval": 100,
        "animation_repeat_delay": 1000,
        "animation_blit": True,
        "title": {"fontsize": 24, "fontweight": "bold", "color": "white"},
    },
    "ASV": {
        "court_line_color": asvel_pink,
        "line_width": 2,
        "outer_lines": True,
        "court_background_color": "black",
        "color_map": {"made": asvel_pink, "miss": asvel_gray},
        "marker_size": 8,
        "figsize": (14, 12),
        #'made_action_ids': ['LAYUPMD', '3FGM', '2FGM', 'DUNK', ],
        #'miss_action_ids': ['3FGA', 'LAYUPATT','2FGA', '3FGAB', '2FGAB'],
        "sort_col": "UTC",
        "coord_x": "COORD_X",
        "coord_y": "COORD_Y",
        "sort_col": "UTC",
        "animation_interval": 100,
        "animation_repeat_delay": 1000,
        "animation_blit": True,
        "title": {"fontsize": 24, "fontweight": "bold", "color": asvel_pink},
    },
    "BER": {
        "court_line_color": alba_light_blue,
        "line_width": 2,
        "outer_lines": True,
        "court_background_color": alba_blue,
        "color_map": {"made": alba_yellow, "miss": alba_light_blue},
        "marker_size": 8,
        "figsize": (14, 12),
        #'made_action_ids': ['LAYUPMD', '3FGM', '2FGM', 'DUNK', ],
        #'miss_action_ids': ['3FGA', 'LAYUPATT','2FGA', '3FGAB', '2FGAB'],
        "sort_col": "UTC",
        "coord_x": "COORD_X",
        "coord_y": "COORD_Y",
        "sort_col": "UTC",
        "animation_interval": 100,
        "animation_repeat_delay": 1000,
        "animation_blit": True,
        "title": {"fontsize": 24, "fontweight": "bold", "color": alba_yellow},
    },
    "PRS": {  # the key might be wrong (we will know when the season starts)
        "court_line_color": "white",
        "line_width": 2,
        "outer_lines": True,
        "court_background_color": paris_black,
        "color_map": {"made": paris_blue, "miss": paris_red},
        "marker_size": 8,
        "figsize": (14, 12),
        #'made_action_ids': ['LAYUPMD', '3FGM', '2FGM', 'DUNK', ],
        #'miss_action_ids': ['3FGA', 'LAYUPATT','2FGA', '3FGAB', '2FGAB'],
        "sort_col": "UTC",
        "coord_x": "COORD_X",
        "coord_y": "COORD_Y",
        "sort_col": "UTC",
        "animation_interval": 100,
        "animation_repeat_delay": 1000,
        "animation_blit": True,
        "title": {"fontsize": 24, "fontweight": "bold", "color": "white"},
    },
    "TEL": {  # the key might be wrong (we will know when the season starts)
        "court_line_color": maccabi_yellow,
        "line_width": 2,
        "outer_lines": True,
        "court_background_color": maccabi_blue,
        "color_map": {"made": maccabi_yellow, "miss": "white"},
        "marker_size": 8,
        "figsize": (14, 12),
        #'made_action_ids': ['LAYUPMD', '3FGM', '2FGM', 'DUNK', ],
        #'miss_action_ids': ['3FGA', 'LAYUPATT','2FGA', '3FGAB', '2FGAB'],
        "sort_col": "UTC",
        "coord_x": "COORD_X",
        "coord_y": "COORD_Y",
        "sort_col": "UTC",
        "animation_interval": 100,
        "animation_repeat_delay": 1000,
        "animation_blit": True,
        "title": {"fontsize": 24, "fontweight": "bold", "color": maccabi_yellow},
    },
}
